import * as PairingHeap from './PairingHeap';
import { PriorityQueueIterator } from './PriorityQueueIterator';
export declare const DEFAULT_COMPARATOR: (a: any, b: any) => number;
export interface Data<T, P> {
    heap: PairingHeap.MaybeHeap<T, P>;
    size: number;
}
export declare const NIL: Data<any, any>;
export declare function make<T, P>(arraylike?: [T, P][], comparator?: (a: P, b: P) => number): Data<T, P>;
export declare function isEmpty<T, P>({size}: Data<T, P>): boolean;
export declare function clear<T, P>(_?: Data<T, P>): Data<T, P>;
export declare function first<T, P>({heap}: Data<T, P>): [T, P] | undefined;
export declare function firstElement<T, P>({heap}: Data<T, P>): T | undefined;
export declare function firstPriority<T, P>({heap}: Data<T, P>): P | undefined;
export declare function enqueue<T, P>({heap, size}: Data<T, P>, element: T, priority: P, comparator?: (a: P, b: P) => number): Data<T, P>;
export declare function dequeue<T, P>(queue: Data<T, P>, comparator?: (a: P, b: P) => number): Data<T, P>;
export declare function merge<T, P>(queue: Data<T, P>, otherQueue: Data<T, P>, comparator?: (a: P, b: P) => number): Data<T, P>;
export declare function has<T, P>(queue: Data<T, P>, element: T): boolean;
export declare function forEach<T, P, Z, Y>(queue: Data<T, P>, callbackFn: (this: Z, element: T, priority: P, callbackArg: Y) => void, thisArg?: Z, callbackArg?: any): void;
export declare function iterator<T, P>(queue: Data<T, P>): PriorityQueueIterator<T, P>;
export declare class PriorityQueue<T, P> {
    protected comparator: (a: P, b: P) => number;
    protected queue: Data<T, P>;
    readonly size: number;
    constructor(arraylike?: [T, P][], comparator?: (a: P, b: P) => number);
    static ofData<T, P>(queue: Data<T, P>, comparator?: (a: P, b: P) => number): PriorityQueue<T, P>;
    isEmpty(): boolean;
    clear(): PriorityQueue<T, P>;
    first(): [T, P] | undefined;
    firstElement(): T | undefined;
    firstPriority(): P | undefined;
    enqueue(element: T, priority: P): PriorityQueue<T, P>;
    dequeue(): PriorityQueue<T, P>;
    merge(queue: PriorityQueue<T, P>): PriorityQueue<T, P>;
    has(element: T): boolean;
    forEach<Z>(callbackFn: (this: Z, element: T, priority: P, pq: this) => void, thisArg?: Z): void;
    iterator(): PriorityQueueIterator<T, P>;
    [Symbol.iterator](): PriorityQueueIterator<T, P>;
}
